/*   EXTRAITS DE LA LICENCE
	Copyright CEA, contributeurs : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2006)
  
	Adresse ml :
	BILLARD, non joignable par ml ;
	CALISTE, damien P caliste AT cea P fr.

	Ce logiciel est un programme informatique servant  visualiser des
	structures atomiques dans un rendu pseudo-3D. 

	Ce logiciel est rgi par la licence CeCILL soumise au droit franais et
	respectant les principes de diffusion des logiciels libres. Vous pouvez
	utiliser, modifier et/ou redistribuer ce programme sous les conditions
	de la licence CeCILL telle que diffuse par le CEA, le CNRS et l'INRIA 
	sur le site "http://www.cecill.info".

	Le fait que vous puissiez accder  cet en-tte signifie que vous avez 
	pris connaissance de la licence CeCILL, et que vous en avez accept les
	termes (cf. le fichier Documentation/licence.fr.txt fourni avec ce logiciel).
*/

/*   LICENCE SUM UP
	Copyright CEA, contributors : Luc BILLARD et Damien
	CALISTE, laboratoire L_Sim, (2001-2006)

	E-mail address:
	BILLARD, not reachable any more ;
	CALISTE, damien P caliste AT cea P fr.

	This software is a computer program whose purpose is to visualize atomic
	configurations in 3D.

	This software is governed by the CeCILL  license under French law and
	abiding by the rules of distribution of free software.  You can  use, 
	modify and/ or redistribute the software under the terms of the CeCILL
	license as circulated by CEA, CNRS and INRIA at the following URL
	"http://www.cecill.info". 

	The fact that you are presently reading this means that you have had
	knowledge of the CeCILL license and that you accept its terms. You can
	find a copy of this licence shipped with this software at Documentation/licence.en.txt.
*/

#include "toolShade.h"

#include <visu_tools.h>
#include <string.h>

struct Shade_struct
{
  gchar* labelUTF8;
  float vectA[3], vectB[3];
  gboolean vectX[3];
  int colorMode;
  gchar* pathToImage;
};

GList *toolShadeList = (GList*)0;

Shade* shadeNew(const gchar* labelUTF8, float vectA[3], float vectB[3],
	       gboolean vectX[3], ShadeColorMode colorMode)
{
  Shade *shade;
  int i;

  g_return_val_if_fail(labelUTF8 && vectA && vectB && vectX &&
		       colorMode < shade_nb_colorMode, (Shade*)0);

  DBG_fprintf(stderr, "Tool Shade : creating a new Shade object :");
  shade = g_malloc(sizeof(Shade));
  shade->labelUTF8 = g_strdup(labelUTF8);
  for (i = 0; i < 3; i++)
    {
      shade->vectA[i] = vectA[i];
      shade->vectB[i] = vectB[i];
      shade->vectX[i] = vectX[i];
    }
  shade->colorMode = colorMode;
  shade->pathToImage = (gchar*)0;

  DBG_fprintf(stderr, " %p.\n", (gpointer)shade);
  return shade;
}

gboolean shadeSet_pathToImage(Shade *shade, const gchar* path)
{
  gchar *absPath;

  g_return_val_if_fail(shade, FALSE);

  if (!g_path_is_absolute(path))
    absPath = g_build_filename(v_sim_pixmaps_dir, path, NULL);
  else
    absPath = g_strdup(path);

  if (shade->pathToImage && strcmp((char*)shade->pathToImage, (char*)absPath) == 0)
    {
      g_free(absPath);
      return FALSE;
    }

  if (shade->pathToImage)
    g_free(shade->pathToImage);

  shade->pathToImage = absPath;

  DBG_fprintf(stderr, "Tool Shade : set pathToImage of shade"
	      " %p to '%s'.\n", (gpointer)shade, absPath);
  return TRUE;
}

gchar* shadeGet_label(Shade *shade)
{
  g_return_val_if_fail(shade, (gchar*)0);
  return shade->labelUTF8;
}
gchar* shadeGet_pathToImage(Shade *shade)
{
  g_return_val_if_fail(shade, (gchar*)0);
  return shade->pathToImage;
}
ShadeColorMode shadeGet_colorMode(Shade *shade)
{
  g_return_val_if_fail(shade, (int)0);
  return shade->colorMode;
}
gboolean shadeGet_colorTransformation(Shade *shade, float **vectA, float **vectB, gboolean **vectX)
{
  g_return_val_if_fail(shade, FALSE);
  g_return_val_if_fail(vectA && vectB && vectX, FALSE);
  
  *vectA = shade->vectA;
  *vectB = shade->vectB;
  *vectX = shade->vectX;
  return TRUE;
}


/*********************************************/
/* Methods to deal with internal shade list. */
/*********************************************/
GList* toolShadeListGet()
{
  return toolShadeList;
}
GList* toolShadeListAppend(Shade *shade)
{
  g_return_val_if_fail(shade, (GList*)0);

  toolShadeList = g_list_append(toolShadeList, (gpointer)shade);
  return toolShadeList;
}
